import math

from pmma.core.py_src.Constants import InternalConstants

class TimeFormatter:
    def __init__(self):
        self._years = 0
        self._months = 0
        self._days = 0
        self._hours = 0
        self._minutes = 0
        self._seconds = 0
        self._microseconds = 0

    def set_from_year(self, year):
        # Convert to total seconds
        total_seconds = year * InternalConstants.DAYS_PER_YEAR * InternalConstants.SECONDS_PER_DAY

        # Years
        years = math.floor(year)
        remainder_seconds = total_seconds - (years * InternalConstants.DAYS_PER_YEAR * InternalConstants.SECONDS_PER_DAY)

        # Months
        month = remainder_seconds / (InternalConstants.DAYS_PER_MONTH * InternalConstants.SECONDS_PER_DAY)
        months = math.floor(month)
        remainder_seconds -= months * InternalConstants.DAYS_PER_MONTH * InternalConstants.SECONDS_PER_DAY

        # Days
        day = remainder_seconds / InternalConstants.SECONDS_PER_DAY
        days = math.floor(day)
        remainder_seconds -= days * InternalConstants.SECONDS_PER_DAY

        # Hours
        hour = remainder_seconds / InternalConstants.SECONDS_PER_HOUR
        hours = math.floor(hour)
        remainder_seconds -= hours * InternalConstants.SECONDS_PER_HOUR

        # Minutes
        minute = remainder_seconds / InternalConstants.SECONDS_PER_MINUTE
        minutes = math.floor(minute)
        remainder_seconds -= minutes * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_month(self, month):
        # Years
        years = math.floor(month / 12)
        months_remainder = month - years * 12

        # Months
        months = math.floor(months_remainder)
        day = (months_remainder - months) * InternalConstants.DAYS_PER_MONTH
        days = math.floor(day)

        # Remainder in seconds
        remainder_seconds = (day - days) * InternalConstants.SECONDS_PER_DAY

        # Hours
        hours = math.floor(remainder_seconds / InternalConstants.SECONDS_PER_HOUR)
        remainder_seconds -= hours * InternalConstants.SECONDS_PER_HOUR

        # Minutes
        minutes = math.floor(remainder_seconds / InternalConstants.SECONDS_PER_MINUTE)
        remainder_seconds -= minutes * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_day(self, day):
        # Years
        years = math.floor(day / InternalConstants.DAYS_PER_YEAR)
        days_remainder = day - years * InternalConstants.DAYS_PER_YEAR

        # Months
        months = math.floor(days_remainder / InternalConstants.DAYS_PER_MONTH)
        days_remainder -= months * InternalConstants.DAYS_PER_MONTH

        # Days
        days = math.floor(days_remainder)

        # Remainder in seconds
        remainder_seconds = (days_remainder - days) * InternalConstants.SECONDS_PER_DAY

        # Hours
        hours = math.floor(remainder_seconds / InternalConstants.SECONDS_PER_HOUR)
        remainder_seconds -= hours * InternalConstants.SECONDS_PER_HOUR

        # Minutes
        minutes = math.floor(remainder_seconds / InternalConstants.SECONDS_PER_MINUTE)
        remainder_seconds -= minutes * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_hour(self, hour):
        # Years
        days_float = hour / InternalConstants.HOURS_PER_DAY
        years = math.floor(days_float / InternalConstants.DAYS_PER_YEAR)
        days_remainder = days_float - years * InternalConstants.DAYS_PER_YEAR

        # Months
        months = math.floor(days_remainder / InternalConstants.DAYS_PER_MONTH)
        days_remainder -= months * InternalConstants.DAYS_PER_MONTH

        # Days
        days = math.floor(days_remainder)
        hours_remainder = (days_remainder - days) * InternalConstants.HOURS_PER_DAY

        # Hours
        hours = math.floor(hours_remainder)
        minute = (hours_remainder - hours) * InternalConstants.SECONDS_PER_HOUR / InternalConstants.SECONDS_PER_MINUTE

        # Minutes
        minutes = math.floor(minute)
        remainder_seconds = (minute - minutes) * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_minute(self, minute):
        # Years
        hours_float = minute / InternalConstants.MINUTES_PER_HOUR
        days_float = hours_float / InternalConstants.HOURS_PER_DAY
        years = math.floor(days_float / InternalConstants.DAYS_PER_YEAR)
        days_remainder = days_float - years * InternalConstants.DAYS_PER_YEAR

        # Months
        months = math.floor(days_remainder / InternalConstants.DAYS_PER_MONTH)
        days_remainder -= months * InternalConstants.DAYS_PER_MONTH

        # Days
        days = math.floor(days_remainder)
        hours_float = (days_remainder - days) * InternalConstants.HOURS_PER_DAY

        # Hours
        hours = math.floor(hours_float)
        minutes_remainder = (hours_float - hours) * InternalConstants.MINUTES_PER_HOUR

        # Minutes
        minutes = math.floor(minutes_remainder)
        remainder_seconds = (minutes_remainder - minutes) * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_second(self, second):
        # Years
        days_float = second / InternalConstants.SECONDS_PER_DAY
        years = math.floor(days_float / InternalConstants.DAYS_PER_YEAR)
        days_remainder = days_float - years * InternalConstants.DAYS_PER_YEAR

        # Months
        months = math.floor(days_remainder / InternalConstants.DAYS_PER_MONTH)
        days_remainder -= months * InternalConstants.DAYS_PER_MONTH

        # Days
        days = math.floor(days_remainder)
        hours_float = (days_remainder - days) * InternalConstants.HOURS_PER_DAY

        # Hours
        hours = math.floor(hours_float)
        minutes_float = (hours_float - hours) * InternalConstants.MINUTES_PER_HOUR

        # Minutes
        minutes = math.floor(minutes_float)
        remainder_seconds = (minutes_float - minutes) * InternalConstants.SECONDS_PER_MINUTE

        # Seconds
        seconds = math.floor(remainder_seconds)
        remainder_seconds -= seconds

        # Microseconds
        microseconds = math.floor(remainder_seconds * 1e6)

        self._years = years
        self._months = months
        self._days = days
        self._hours = hours
        self._minutes = minutes
        self._seconds = seconds
        self._microseconds = microseconds

    def set_from_microsecond(self, microsecond):
        # Convert microseconds to seconds
        seconds_float = microsecond / InternalConstants.MICROSECONDS_PER_SECOND

        # Use the seconds conversion function
        return self.set_from_second(seconds_float)

    def get_year(self):
        return self._years

    def get_month(self):
        return self._months

    def get_day(self):
        return self._days

    def get_hour(self):
        return self._hours

    def get_minute(self):
        return self._minutes

    def get_second(self):
        return self._seconds

    def get_microsecond(self):
        return self._microseconds

    def get_in_date_format(self):
        return f"{self._days}/{self._months}/{self._years} @ {self._hours}:{self._minutes}:{self._seconds}.{self._microseconds}"

    def get_in_sentence_format(self):
        output_string = ""
        if self._years > 0:
            if self._years > 1:
                output_string += f"{self._years} years, "
            else:
                output_string += f"{self._years} year, "

        if self._months > 0:
            if self._months > 1:
                output_string += f"{self._months} months, "
            else:
                output_string += f"{self._months} month, "

        if self._days > 0:
            if self._days > 1:
                output_string += f"{self._days} days, "
            else:
                output_string += f"{self._days} day, "

        if self._hours > 0:
            if self._hours > 1:
                output_string += f"{self._hours} hours, "
            else:
                output_string += f"{self._hours} hour, "

        if self._minutes > 0:
            if self._minutes > 1:
                output_string += f"{self._minutes} minutes, "
            else:
                output_string += f"{self._minutes} minute, "

        if self._seconds > 0:
            if self._seconds > 1:
                output_string += f"{self._seconds} seconds, "
            else:
                output_string += f"{self._seconds} second, "

        if self._microseconds > 0:
            if self._microseconds > 1:
                output_string += f"{self._microseconds} microseconds.  "
            else:
                output_string += f"{self._microseconds} microsecond.  "

        output_string = " and ".join(output_string[:-2].rsplit(", ", 1))
        return output_string


from typing import Union, Iterable

import numpy
from numpy.typing import NDArray

NoneInt = Union[int, None]
NoneFloat = Union[float, None]

class Sampler:
    def __init__(self, chunk_size: int=2048, sampling_rate: int=44100, input_device_id: NoneInt=None) -> None: ...
    def print_input_devices(self) -> None: ...
    def sampler(self, wait_time: NoneFloat) -> None: ...
    def start(self, wait_time: NoneFloat=None) -> None: ...
    def stop(self, wait_until_stopped: bool=True) -> None: ...
    def pause(self) -> None: ...
    def unpause(self) -> None: ...
    def set_input_device(self, input_device_id: NoneInt=None) -> None: ...

    def get_input_devices(self) -> Iterable[int]: ...

    def get_default_input_device(self) -> int: ...
    def get_loudest_frequency(self) -> int: ...

    def get_frequency(self) -> NDArray[numpy.float64]: ...

    def get_volume(self) -> float: ...

    def is_sampling(self) -> bool: ...


from typing import Iterable, Union

import numpy as np
import numpy.typing as npt

import pmma.core.py_src.Utility as Utility
from pmma.build.NumberFormats import Color

Integer1D = Union[
    npt.NDArray[np.int32], # preferred
    npt.NDArray[np.int8],
    npt.NDArray[np.int16],
    npt.NDArray[np.int64],
    Iterable[int],
]

OptionalInteger = Union[
    int,
    None,
]

OptionalColor = Union[
    Color,
    None
]

OptionalBool = Union[
    bool,
    None
]

Float1D = Union[
    npt.NDArray[np.float32], # preferred
    npt.NDArray[np.float16],
    npt.NDArray[np.float64],
    Iterable[float]]

Numerical = Union[float, int]

NoneInt = Union[int, None]

class Display:
    def __init__(self, seed: OptionalInteger=None, octaves: int=2, lacunarity: float=0.75, gain: float=1.0) -> None: ...

    window_fill_color: Color

    def create(self, size: Integer1D=..., caption: str="PMMA Display", fullscreen: OptionalBool=True, resizable: bool=False, no_frame: bool=False, vsync: bool = True, icon: str="", centered: bool=True, maximized: bool=False, transparent: bool=False) -> None: ...

    def center_window(self) -> None: ...

    @Utility.require_render_thread
    def clear(self) -> None: ...

    def set_window_in_focus(self) -> None: ...
    def set_window_minimized(self, value: bool) -> None: ...
    def set_window_maximized(self, value: bool) -> None: ...
    def set_icon(self, icon_path: str) -> None: ...
    def set_caption(self, caption: str) -> None: ...
    def set_relative_window_position(self, position: Integer1D) -> None: ...
    def set_absolute_window_position(self, position: Integer1D) -> None: ...

    @Utility.require_render_thread
    def continuous_refresh(self,
        refresh_rate: int=60,
        lower_refresh_rate_on_minimize: bool=True,
        lower_refresh_rate_on_focus_loss: bool=True,
        lower_refresh_rate_on_low_battery: bool=True) -> None: ...

    @Utility.require_render_thread
    def event_refresh(self, refresh_rate: int=60,
        max_refresh_rate: int=60,
        lower_refresh_rate_on_minimize: bool=True,
        lower_refresh_rate_on_focus_loss: bool=True,
        lower_refresh_rate_on_low_battery: bool=True) -> None: ...

    def trigger_event_refresh(self) -> None: ...

    def get_caption(self) -> str: ...
    def get_center(self, object_size: Union[None, Integer1D]=None) -> Integer1D: ...
    def get_aspect_ratio(self) -> float: ...
    def get_frame_rate(self) -> int: ...
    def get_frame_time(self) -> float: ...
    def get_width(self) -> int: ...
    def get_height(self) -> int: ...
    def get_size(self) -> Integer1D: ...
    def get_msaa_samples(self) -> int: ...
    def get_current_monitor_refresh_rate(self) -> int: ...

    def toggle_full_screen(self) -> None: ...

    def is_window_in_focus(self) -> bool: ...
    def is_window_minimized(self) -> bool: ...
    def is_window_resizable(self) -> bool: ...
    def is_window_visible(self) -> bool: ...
    def is_window_always_on_top(self) -> bool: ...
    def is_window_auto_minimized(self) -> bool: ...
    def is_window_maximized(self) -> bool: ...
    def is_window_using_vsync(self) -> int: ...


from typing import Iterable, Union

class GPUs:
    def __init__(self) -> None: ...

    def identify_gpus(self) -> None: ...

    def get_gpu(self, gpu_index: int) -> GPU: ...

    def all_gpus_are_unique(self) -> bool: ...

class GPU:
    def __init__(self, module_identification_indices: int) -> None: ...

    def update(self, everything: bool=False, data_points: Union[Iterable[str], None]=None, wait_for_completion: bool=False) -> None: ...

    def get_accelerator_capabilities(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_accounting_mode_enabled(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_accounting_mode_buffer_size(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_adapter_compatibility(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_adapter_DAC_type(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_adapter_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_adapter_index(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_addressing_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_availability(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_capability_descriptions(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_caption(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_chip_to_chip_interconnect_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_as_bitmap(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_application_setting(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_is_hardware_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_gpu_idle_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_software_power_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_software_thermal_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_power_break_slowdown_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_supported(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_sync_boost(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_clock_event_reasons_thermal_limited(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_color_table_entries(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_compute_cap(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_compute_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_config_manager_error_code(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_config_manager_user_config(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_core_voltage(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_core_voltage_range(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_creation_class_name(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_bits_per_pixel(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_horizontal_resolution(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_number_of_colors(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_number_of_columns(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_number_of_rows(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_refresh_rate(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_scan_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_current_vertical_resolution(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_description(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_device_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_device_specific_pens(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_display_active(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_display_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_dither_type(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_driver_date(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_driver_model_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_driver_model_pending(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_driver_version(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_cbu(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_primary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_register_file(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_secondary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_shared_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_sram(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_texture_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_all_time_in_video_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_cbu(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_primary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_register_file(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_secondary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_shared_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_sram(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_texture_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_corrected_since_reboot_in_video_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_cbu(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_primary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_register_file(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_secondary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_shared_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_sram(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_texture_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_all_time_in_video_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_cbu(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_primary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_register_file(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_secondary_cache(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_shared_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_sram(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_texture_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_errors_uncorrected_since_reboot_in_video_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_mode_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ecc_mode_pending(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_encoder_average_FPS(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_encoder_average_latency(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_encoder_session_count(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_engine_clock_range(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_error_cleared(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_error_description(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fabric_state(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fabric_status(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fan_speed_percentage(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fan_speed_percentage_range(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fan_speed_RPM(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fan_speed_RPM_range(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_fractional_multi_vGPU(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_application_default_shader_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_application_default_memory_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_application_memory_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_application_shader_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_maximum_memory_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_maximum_shader_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_maximum_streaming_multiprocessor_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_memory_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_shader_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_streaming_multiprocessor_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_frequency_video_clock(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_heterogenous_multi_vGPU(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_heterogenous_time_slice_profile(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_heterogenous_time_slice_sizes(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ICM_indent(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_ICM_method(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_inf_filename(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_inf_section(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_info_ROM_ecc(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_info_ROM_oem(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_info_ROM_power(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_info_ROM_version(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_install_date(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_installed_display_drivers(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_last_error_code(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_max_memory_supported(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_max_number_controlled(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_max_refresh_rate(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_memory_clock_range(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_memory_free(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_memory_reserved(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_memory_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_memory_used(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_min_refresh_rate(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_monochrome(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_multi_instance_GPU_mode_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_multi_instance_GPU_mode_pending(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_name(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_number_of_color_planes(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_number_of_video_pages(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_operating_mode_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_operating_mode_pending(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_bus(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_bus_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_device(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_device_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_domain(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_generation_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_generation_device_host_maximum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_generation_gpu_maximum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_generation_maximum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_width_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_link_width_maximum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_pci_sub_device_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_persistence_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_PNP_device_id(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_average(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_default_limit(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_enforced_limit(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_instant(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_limit(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_maximum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_draw_minimum(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_management_capabilities(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_power_management_supported(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_protected_memory_free(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_protected_memory_total(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_protected_memory_used(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_protocol_supported(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_performance_state(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_retired_pages_double_bit_ecc_errors_count(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_retired_pages_single_bit_ecc_errors_count(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_retired_pages_pending(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_reserved_system_palette_entries(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_reset_required(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_reset_and_drain_recommended(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_serial(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_specification_version(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_status(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_status_info(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_system_creation_class_name(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_system_name(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_system_palette_entries(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_GPU_system_processor_mode_current(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_GPU_system_processor_mode_default(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_temperature_core(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_temperature_core_limit(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_temperature_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_time_of_last_reset(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_decoder(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_encoder(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_gpu(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_jpeg(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_memory(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_utilization_optical_flow(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_uuid(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_vbios_version(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_video_architecture(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_video_memory_type(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_video_mode(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_video_mode_description(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def get_video_processor(self, update: bool=False, wait_for_completion: bool=False) -> str: ...

    def set_accelerator_capabilities(self, value: Union[str, None]=None) -> None: ...

    def set_accounting_mode_enabled(self, value: Union[str, None]=None) -> None: ...

    def set_accounting_mode_buffer_size(self, value: Union[str, None]=None) -> None: ...

    def set_adapter_compatibility(self, value: Union[str, None]=None) -> None: ...

    def set_adapter_DAC_type(self, value: Union[str, None]=None) -> None: ...

    def set_adapter_id(self, value: Union[str, None]=None) -> None: ...

    def set_adapter_index(self, value: Union[str, None]=None) -> None: ...

    def set_addressing_mode(self, value: Union[str, None]=None) -> None: ...

    def set_availability(self, value: Union[str, None]=None) -> None: ...

    def set_capability_descriptions(self, value: Union[str, None]=None) -> None: ...

    def set_caption(self, value: Union[str, None]=None) -> None: ...

    def set_chip_to_chip_interconnect_mode(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_as_bitmap(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_application_setting(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_is_hardware_limited(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_gpu_idle_limited(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_software_power_limited(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_software_thermal_limited(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_power_break_slowdown_limited(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_supported(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_sync_boost(self, value: Union[str, None]=None) -> None: ...

    def set_clock_event_reasons_thermal_limited(self, value: Union[str, None]=None) -> None: ...

    def set_color_table_entries(self, value: Union[str, None]=None) -> None: ...

    def set_compute_cap(self, value: Union[str, None]=None) -> None: ...

    def set_compute_mode(self, value: Union[str, None]=None) -> None: ...

    def set_config_manager_error_code(self, value: Union[str, None]=None) -> None: ...

    def set_config_manager_user_config(self, value: Union[str, None]=None) -> None: ...

    def set_core_voltage(self, value: Union[str, None]=None) -> None: ...

    def set_core_voltage_range(self, value: Union[str, None]=None) -> None: ...

    def set_creation_class_name(self, value: Union[str, None]=None) -> None: ...

    def set_current_bits_per_pixel(self, value: Union[str, None]=None) -> None: ...

    def set_current_horizontal_resolution(self, value: Union[str, None]=None) -> None: ...

    def set_current_number_of_colors(self, value: Union[str, None]=None) -> None: ...

    def set_current_number_of_columns(self, value: Union[str, None]=None) -> None: ...

    def set_current_number_of_rows(self, value: Union[str, None]=None) -> None: ...

    def set_current_refresh_rate(self, value: Union[str, None]=None) -> None: ...

    def set_current_scan_mode(self, value: Union[str, None]=None) -> None: ...

    def set_current_vertical_resolution(self, value: Union[str, None]=None) -> None: ...

    def set_description(self, value: Union[str, None]=None) -> None: ...

    def set_device_id(self, value: Union[str, None]=None) -> None: ...

    def set_device_specific_pens(self, value: Union[str, None]=None) -> None: ...

    def set_display_active(self, value: Union[str, None]=None) -> None: ...

    def set_display_mode(self, value: Union[str, None]=None) -> None: ...

    def set_dither_type(self, value: Union[str, None]=None) -> None: ...

    def set_driver_date(self, value: Union[str, None]=None) -> None: ...

    def set_driver_model_current(self, value: Union[str, None]=None) -> None: ...

    def set_driver_model_pending(self, value: Union[str, None]=None) -> None: ...

    def set_driver_version(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_cbu(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_primary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_register_file(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_secondary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_shared_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_sram(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_texture_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_total(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_all_time_in_video_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_cbu(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_primary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_register_file(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_secondary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_shared_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_sram(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_texture_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_total(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_corrected_since_reboot_in_video_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_cbu(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_primary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_register_file(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_secondary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_shared_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_sram(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_texture_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_total(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_all_time_in_video_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_cbu(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_primary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_register_file(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_secondary_cache(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_shared_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_sram(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_texture_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_total(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_errors_uncorrected_since_reboot_in_video_memory(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_mode_current(self, value: Union[str, None]=None) -> None: ...

    def set_ecc_mode_pending(self, value: Union[str, None]=None) -> None: ...

    def set_encoder_average_FPS(self, value: Union[str, None]=None) -> None: ...

    def set_encoder_average_latency(self, value: Union[str, None]=None) -> None: ...

    def set_encoder_session_count(self, value: Union[str, None]=None) -> None: ...

    def set_engine_clock_range(self, value: Union[str, None]=None) -> None: ...

    def set_error_cleared(self, value: Union[str, None]=None) -> None: ...

    def set_error_description(self, value: Union[str, None]=None) -> None: ...

    def set_fabric_state(self, value: Union[str, None]=None) -> None: ...

    def set_fabric_status(self, value: Union[str, None]=None) -> None: ...

    def set_fan_speed_percentage(self, value: Union[str, None]=None) -> None: ...

    def set_fan_speed_percentage_range(self, value: Union[str, None]=None) -> None: ...

    def set_fan_speed_RPM(self, value: Union[str, None]=None) -> None: ...

    def set_fan_speed_RPM_range(self, value: Union[str, None]=None) -> None: ...

    def set_fractional_multi_vGPU(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_application_default_shader_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_application_default_memory_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_application_memory_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_application_shader_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_maximum_memory_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_maximum_shader_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_maximum_streaming_multiprocessor_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_memory_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_shader_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_streaming_multiprocessor_clock(self, value: Union[str, None]=None) -> None: ...

    def set_frequency_video_clock(self, value: Union[str, None]=None) -> None: ...

    def set_heterogenous_multi_vGPU(self, value: Union[str, None]=None) -> None: ...

    def set_heterogenous_time_slice_profile(self, value: Union[str, None]=None) -> None: ...

    def set_heterogenous_time_slice_sizes(self, value: Union[str, None]=None) -> None: ...

    def set_ICM_indent(self, value: Union[str, None]=None) -> None: ...

    def set_ICM_method(self, value: Union[str, None]=None) -> None: ...

    def set_inf_filename(self, value: Union[str, None]=None) -> None: ...

    def set_inf_section(self, value: Union[str, None]=None) -> None: ...

    def set_info_ROM_ecc(self, value: Union[str, None]=None) -> None: ...

    def set_info_ROM_oem(self, value: Union[str, None]=None) -> None: ...

    def set_info_ROM_power(self, value: Union[str, None]=None) -> None: ...

    def set_info_ROM_version(self, value: Union[str, None]=None) -> None: ...

    def set_install_date(self, value: Union[str, None]=None) -> None: ...

    def set_installed_display_drivers(self, value: Union[str, None]=None) -> None: ...

    def set_last_error_code(self, value: Union[str, None]=None) -> None: ...

    def set_max_memory_supported(self, value: Union[str, None]=None) -> None: ...

    def set_max_number_controlled(self, value: Union[str, None]=None) -> None: ...

    def set_max_refresh_rate(self, value: Union[str, None]=None) -> None: ...

    def set_memory_clock_range(self, value: Union[str, None]=None) -> None: ...

    def set_memory_free(self, value: Union[str, None]=None) -> None: ...

    def set_memory_reserved(self, value: Union[str, None]=None) -> None: ...

    def set_memory_total(self, value: Union[str, None]=None) -> None: ...

    def set_memory_used(self, value: Union[str, None]=None) -> None: ...

    def set_min_refresh_rate(self, value: Union[str, None]=None) -> None: ...

    def set_monochrome(self, value: Union[str, None]=None) -> None: ...

    def set_multi_instance_GPU_mode_current(self, value: Union[str, None]=None) -> None: ...

    def set_multi_instance_GPU_mode_pending(self, value: Union[str, None]=None) -> None: ...

    def set_name(self, value: Union[str, None]=None) -> None: ...

    def set_number_of_color_planes(self, value: Union[str, None]=None) -> None: ...

    def set_number_of_video_pages(self, value: Union[str, None]=None) -> None: ...

    def set_operating_mode_current(self, value: Union[str, None]=None) -> None: ...

    def set_operating_mode_pending(self, value: Union[str, None]=None) -> None: ...

    def set_pci_bus(self, value: Union[str, None]=None) -> None: ...

    def set_pci_bus_id(self, value: Union[str, None]=None) -> None: ...

    def set_pci_device(self, value: Union[str, None]=None) -> None: ...

    def set_pci_device_id(self, value: Union[str, None]=None) -> None: ...

    def set_pci_domain(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_generation_current(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_generation_device_host_maximum(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_generation_gpu_maximum(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_generation_maximum(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_width_current(self, value: Union[str, None]=None) -> None: ...

    def set_pci_link_width_maximum(self, value: Union[str, None]=None) -> None: ...

    def set_pci_sub_device_id(self, value: Union[str, None]=None) -> None: ...

    def set_persistence_mode(self, value: Union[str, None]=None) -> None: ...

    def set_PNP_device_id(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_average(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_default_limit(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_enforced_limit(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_instant(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_limit(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_maximum(self, value: Union[str, None]=None) -> None: ...

    def set_power_draw_minimum(self, value: Union[str, None]=None) -> None: ...

    def set_power_management_capabilities(self, value: Union[str, None]=None) -> None: ...

    def set_power_management_supported(self, value: Union[str, None]=None) -> None: ...

    def set_protected_memory_free(self, value: Union[str, None]=None) -> None: ...

    def set_protected_memory_total(self, value: Union[str, None]=None) -> None: ...

    def set_protected_memory_used(self, value: Union[str, None]=None) -> None: ...

    def set_protocol_supported(self, value: Union[str, None]=None) -> None: ...

    def set_performance_state(self, value: Union[str, None]=None) -> None: ...

    def set_retired_pages_double_bit_ecc_errors_count(self, value: Union[str, None]=None) -> None: ...

    def set_retired_pages_single_bit_ecc_errors_count(self, value: Union[str, None]=None) -> None: ...

    def set_retired_pages_pending(self, value: Union[str, None]=None) -> None: ...

    def set_reserved_system_palette_entries(self, value: Union[str, None]=None) -> None: ...

    def set_reset_required(self, value: Union[str, None]=None) -> None: ...

    def set_reset_and_drain_recommended(self, value: Union[str, None]=None) -> None: ...

    def set_serial(self, value: Union[str, None]=None) -> None: ...

    def set_specification_version(self, value: Union[str, None]=None) -> None: ...

    def set_status(self, value: Union[str, None]=None) -> None: ...

    def set_status_info(self, value: Union[str, None]=None) -> None: ...

    def set_system_creation_class_name(self, value: Union[str, None]=None) -> None: ...

    def set_system_name(self, value: Union[str, None]=None) -> None: ...

    def set_system_palette_entries(self, value: Union[str, None]=None) -> None: ...

    def set_GPU_system_processor_mode_current(self, value: Union[str, None]=None) -> None: ...

    def set_GPU_system_processor_mode_default(self, value: Union[str, None]=None) -> None: ...

    def set_temperature_core(self, value: Union[str, None]=None) -> None: ...

    def set_temperature_core_limit(self, value: Union[str, None]=None) -> None: ...

    def set_temperature_memory(self, value: Union[str, None]=None) -> None: ...

    def set_time_of_last_reset(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_decoder(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_encoder(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_gpu(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_jpeg(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_memory(self, value: Union[str, None]=None) -> None: ...

    def set_utilization_optical_flow(self, value: Union[str, None]=None) -> None: ...

    def set_uuid(self, value: Union[str, None]=None) -> None: ...

    def set_vbios_version(self, value: Union[str, None]=None) -> None: ...

    def set_video_architecture(self, value: Union[str, None]=None) -> None: ...

    def set_video_memory_type(self, value: Union[str, None]=None) -> None: ...

    def set_video_mode(self, value: Union[str, None]=None) -> None: ...

    def set_video_mode_description(self, value: Union[str, None]=None) -> None: ...

    def set_video_processor(self, value: Union[str, None]=None) -> None: ...

    def update_accelerator_capabilities(self, wait_for_completion: bool=True) -> None: ...

    def update_accounting_mode_enabled(self, wait_for_completion: bool=True) -> None: ...

    def update_accounting_mode_buffer_size(self, wait_for_completion: bool=True) -> None: ...

    def update_adapter_compatibility(self, wait_for_completion: bool=True) -> None: ...

    def update_adapter_DAC_type(self, wait_for_completion: bool=True) -> None: ...

    def update_adapter_id(self, wait_for_completion: bool=True) -> None: ...

    def update_adapter_index(self, wait_for_completion: bool=True) -> None: ...

    def update_addressing_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_availability(self, wait_for_completion: bool=True) -> None: ...

    def update_capability_descriptions(self, wait_for_completion: bool=True) -> None: ...

    def update_caption(self, wait_for_completion: bool=True) -> None: ...

    def update_chip_to_chip_interconnect_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_as_bitmap(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_application_setting(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_is_hardware_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_gpu_idle_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_software_power_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_software_thermal_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_power_break_slowdown_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_supported(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_sync_boost(self, wait_for_completion: bool=True) -> None: ...

    def update_clock_event_reasons_thermal_limited(self, wait_for_completion: bool=True) -> None: ...

    def update_color_table_entries(self, wait_for_completion: bool=True) -> None: ...

    def update_compute_cap(self, wait_for_completion: bool=True) -> None: ...

    def update_compute_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_config_manager_error_code(self, wait_for_completion: bool=True) -> None: ...

    def update_config_manager_user_config(self, wait_for_completion: bool=True) -> None: ...

    def update_core_voltage(self, wait_for_completion: bool=True) -> None: ...

    def update_core_voltage_range(self, wait_for_completion: bool=True) -> None: ...

    def update_creation_class_name(self, wait_for_completion: bool=True) -> None: ...

    def update_current_bits_per_pixel(self, wait_for_completion: bool=True) -> None: ...

    def update_current_horizontal_resolution(self, wait_for_completion: bool=True) -> None: ...

    def update_current_number_of_colors(self, wait_for_completion: bool=True) -> None: ...

    def update_current_number_of_columns(self, wait_for_completion: bool=True) -> None: ...

    def update_current_number_of_rows(self, wait_for_completion: bool=True) -> None: ...

    def update_current_refresh_rate(self, wait_for_completion: bool=True) -> None: ...

    def update_current_scan_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_current_vertical_resolution(self, wait_for_completion: bool=True) -> None: ...

    def update_description(self, wait_for_completion: bool=True) -> None: ...

    def update_device_id(self, wait_for_completion: bool=True) -> None: ...

    def update_device_specific_pens(self, wait_for_completion: bool=True) -> None: ...

    def update_display_active(self, wait_for_completion: bool=True) -> None: ...

    def update_display_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_dither_type(self, wait_for_completion: bool=True) -> None: ...

    def update_driver_date(self, wait_for_completion: bool=True) -> None: ...

    def update_driver_model_current(self, wait_for_completion: bool=True) -> None: ...

    def update_driver_model_pending(self, wait_for_completion: bool=True) -> None: ...

    def update_driver_version(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_cbu(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_primary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_register_file(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_secondary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_shared_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_sram(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_texture_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_total(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_all_time_in_video_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_cbu(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_primary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_register_file(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_secondary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_shared_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_sram(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_texture_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_total(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_corrected_since_reboot_in_video_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_cbu(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_primary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_register_file(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_secondary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_shared_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_sram(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_texture_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_total(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_all_time_in_video_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_cbu(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_primary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_register_file(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_secondary_cache(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_shared_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_sram(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_texture_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_total(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_errors_uncorrected_since_reboot_in_video_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_mode_current(self, wait_for_completion: bool=True) -> None: ...

    def update_ecc_mode_pending(self, wait_for_completion: bool=True) -> None: ...

    def update_encoder_average_FPS(self, wait_for_completion: bool=True) -> None: ...

    def update_encoder_average_latency(self, wait_for_completion: bool=True) -> None: ...

    def update_encoder_session_count(self, wait_for_completion: bool=True) -> None: ...

    def update_engine_clock_range(self, wait_for_completion: bool=True) -> None: ...

    def update_error_cleared(self, wait_for_completion: bool=True) -> None: ...

    def update_error_description(self, wait_for_completion: bool=True) -> None: ...

    def update_fabric_state(self, wait_for_completion: bool=True) -> None: ...

    def update_fabric_status(self, wait_for_completion: bool=True) -> None: ...

    def update_fan_speed_percentage(self, wait_for_completion: bool=True) -> None: ...

    def update_fan_speed_percentage_range(self, wait_for_completion: bool=True) -> None: ...

    def update_fan_speed_RPM(self, wait_for_completion: bool=True) -> None: ...

    def update_fan_speed_RPM_range(self, wait_for_completion: bool=True) -> None: ...

    def update_fractional_multi_vGPU(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_application_default_shader_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_application_default_memory_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_application_memory_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_application_shader_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_maximum_memory_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_maximum_shader_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_maximum_streaming_multiprocessor_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_memory_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_shader_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_streaming_multiprocessor_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_frequency_video_clock(self, wait_for_completion: bool=True) -> None: ...

    def update_heterogenous_multi_vGPU(self, wait_for_completion: bool=True) -> None: ...

    def update_heterogenous_time_slice_profile(self, wait_for_completion: bool=True) -> None: ...

    def update_heterogenous_time_slice_sizes(self, wait_for_completion: bool=True) -> None: ...

    def update_ICM_indent(self, wait_for_completion: bool=True) -> None: ...

    def update_ICM_method(self, wait_for_completion: bool=True) -> None: ...

    def update_inf_filename(self, wait_for_completion: bool=True) -> None: ...

    def update_inf_section(self, wait_for_completion: bool=True) -> None: ...

    def update_info_ROM_ecc(self, wait_for_completion: bool=True) -> None: ...

    def update_info_ROM_oem(self, wait_for_completion: bool=True) -> None: ...

    def update_info_ROM_power(self, wait_for_completion: bool=True) -> None: ...

    def update_info_ROM_version(self, wait_for_completion: bool=True) -> None: ...

    def update_install_date(self, wait_for_completion: bool=True) -> None: ...

    def update_installed_display_drivers(self, wait_for_completion: bool=True) -> None: ...

    def update_last_error_code(self, wait_for_completion: bool=True) -> None: ...

    def update_max_memory_supported(self, wait_for_completion: bool=True) -> None: ...

    def update_max_number_controlled(self, wait_for_completion: bool=True) -> None: ...

    def update_max_refresh_rate(self, wait_for_completion: bool=True) -> None: ...

    def update_memory_clock_range(self, wait_for_completion: bool=True) -> None: ...

    def update_memory_free(self, wait_for_completion: bool=True) -> None: ...

    def update_memory_reserved(self, wait_for_completion: bool=True) -> None: ...

    def update_memory_total(self, wait_for_completion: bool=True) -> None: ...

    def update_memory_used(self, wait_for_completion: bool=True) -> None: ...

    def update_min_refresh_rate(self, wait_for_completion: bool=True) -> None: ...

    def update_monochrome(self, wait_for_completion: bool=True) -> None: ...

    def update_multi_instance_GPU_mode_current(self, wait_for_completion: bool=True) -> None: ...

    def update_multi_instance_GPU_mode_pending(self, wait_for_completion: bool=True) -> None: ...

    def update_name(self, wait_for_completion: bool=True) -> None: ...

    def update_number_of_color_planes(self, wait_for_completion: bool=True) -> None: ...

    def update_number_of_video_pages(self, wait_for_completion: bool=True) -> None: ...

    def update_operating_mode_current(self, wait_for_completion: bool=True) -> None: ...

    def update_operating_mode_pending(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_bus(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_bus_id(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_device(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_device_id(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_domain(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_generation_current(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_generation_device_host_maximum(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_generation_gpu_maximum(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_generation_maximum(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_width_current(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_link_width_maximum(self, wait_for_completion: bool=True) -> None: ...

    def update_pci_sub_device_id(self, wait_for_completion: bool=True) -> None: ...

    def update_persistence_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_PNP_device_id(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_average(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_default_limit(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_enforced_limit(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_instant(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_limit(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_maximum(self, wait_for_completion: bool=True) -> None: ...

    def update_power_draw_minimum(self, wait_for_completion: bool=True) -> None: ...

    def update_power_management_capabilities(self, wait_for_completion: bool=True) -> None: ...

    def update_power_management_supported(self, wait_for_completion: bool=True) -> None: ...

    def update_protected_memory_free(self, wait_for_completion: bool=True) -> None: ...

    def update_protected_memory_total(self, wait_for_completion: bool=True) -> None: ...

    def update_protected_memory_used(self, wait_for_completion: bool=True) -> None: ...

    def update_protocol_supported(self, wait_for_completion: bool=True) -> None: ...

    def update_performance_state(self, wait_for_completion: bool=True) -> None: ...

    def update_retired_pages_double_bit_ecc_errors_count(self, wait_for_completion: bool=True) -> None: ...

    def update_retired_pages_single_bit_ecc_errors_count(self, wait_for_completion: bool=True) -> None: ...

    def update_retired_pages_pending(self, wait_for_completion: bool=True) -> None: ...

    def update_reserved_system_palette_entries(self, wait_for_completion: bool=True) -> None: ...

    def update_reset_required(self, wait_for_completion: bool=True) -> None: ...

    def update_reset_and_drain_recommended(self, wait_for_completion: bool=True) -> None: ...

    def update_serial(self, wait_for_completion: bool=True) -> None: ...

    def update_specification_version(self, wait_for_completion: bool=True) -> None: ...

    def update_status(self, wait_for_completion: bool=True) -> None: ...

    def update_status_info(self, wait_for_completion: bool=True) -> None: ...

    def update_system_creation_class_name(self, wait_for_completion: bool=True) -> None: ...

    def update_system_name(self, wait_for_completion: bool=True) -> None: ...

    def update_system_palette_entries(self, wait_for_completion: bool=True) -> None: ...

    def update_GPU_system_processor_mode_current(self, wait_for_completion: bool=True) -> None: ...

    def update_GPU_system_processor_mode_default(self, wait_for_completion: bool=True) -> None: ...

    def update_temperature_core(self, wait_for_completion: bool=True) -> None: ...

    def update_temperature_core_limit(self, wait_for_completion: bool=True) -> None: ...

    def update_temperature_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_time_of_last_reset(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_decoder(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_encoder(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_gpu(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_jpeg(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_memory(self, wait_for_completion: bool=True) -> None: ...

    def update_utilization_optical_flow(self, wait_for_completion: bool=True) -> None: ...

    def update_uuid(self, wait_for_completion: bool=True) -> None: ...

    def update_vbios_version(self, wait_for_completion: bool=True) -> None: ...

    def update_video_architecture(self, wait_for_completion: bool=True) -> None: ...

    def update_video_memory_type(self, wait_for_completion: bool=True) -> None: ...

    def update_video_mode(self, wait_for_completion: bool=True) -> None: ...

    def update_video_mode_description(self, wait_for_completion: bool=True) -> None: ...

    def update_video_processor(self, wait_for_completion: bool=True) -> None: ...



from typing import Literal, Union, Callable, Any

import pedalboard
from moviepy.audio.io.AudioFileClip import AudioFileClip

from pmma.build.NumberConverter import ProportionConverter, LinkedProportionConverter

Numerical = Union[float, int]

class Audio:
    volume: ProportionConverter  # Default volume is 100%
    pan: ProportionConverter  # Pan: -1 (left) to 1 (right), 0 is center

    def __init__(self) -> None: ...

    def __del__(self) -> None: ...

    def set_custom_audio_callback_function_pre_effects(self, function: Union[Callable[..., Any], None]=None) -> None: ...

    def set_custom_audio_callback_function_post_effects(self, function: Union[Callable[..., Any], None]=None) -> None: ...

    def get_custom_audio_callback_function_pre_effects_results(self) -> Any: ...

    def get_custom_audio_callback_function_post_effects_results(self) -> Any: ...

    def load_from_file(self, file_path: str) -> None: ...

    def get_sample_rate(self) -> int: ...

    def get_number_of_channels(self) -> int: ...

    def set_looping(self, loop: bool) -> None: ...

    def get_looping(self) -> bool: ...

    def set_duration(self, duration: Numerical) -> None: ...

    def set_silence_at_end_of_track(self, duration: Numerical) -> None: ...

    def load_from_moviepy(self, audio: AudioFileClip) -> None: ...

    def add_effect(
            self,
            effect: Union[Chorus, Compressor,
                            Convolution, Delay, Phaser,
                            Reverb, pedalboard.Bitcrush,
                            pedalboard.Chorus, pedalboard.Clipping,
                            pedalboard.Compressor,
                            pedalboard.Convolution, pedalboard.Delay,
                            pedalboard.Distortion,
                            pedalboard.GSMFullRateCompressor,
                            pedalboard.Gain, pedalboard.HighShelfFilter,
                            pedalboard.HighpassFilter,
                            pedalboard.LadderFilter, pedalboard.Limiter,
                            pedalboard.LowShelfFilter,
                            pedalboard.LowpassFilter,
                            pedalboard.MP3Compressor,
                            pedalboard.NoiseGate, pedalboard.PeakFilter,
                            pedalboard.Phaser, pedalboard.PitchShift,
                            pedalboard.Resample,
                            pedalboard.Reverb]) -> None: ...

    def remove_effect(
            self,
            effect: Union[Chorus, Compressor,
                            Convolution, Delay, Phaser,
                            Reverb, pedalboard.Bitcrush,
                            pedalboard.Chorus, pedalboard.Clipping,
                            pedalboard.Compressor,
                            pedalboard.Convolution, pedalboard.Delay,
                            pedalboard.Distortion,
                            pedalboard.GSMFullRateCompressor,
                            pedalboard.Gain, pedalboard.HighShelfFilter,
                            pedalboard.HighpassFilter,
                            pedalboard.LadderFilter, pedalboard.Limiter,
                            pedalboard.LowShelfFilter,
                            pedalboard.LowpassFilter,
                            pedalboard.MP3Compressor,
                            pedalboard.NoiseGate, pedalboard.PeakFilter,
                            pedalboard.Phaser, pedalboard.PitchShift,
                            pedalboard.Resample,
                            pedalboard.Reverb]) -> None: ...

    def play(self, blocking: bool=True, delay: Numerical=0) -> bool: ...

    def pause(self) -> None: ...

    def resume(self) -> None: ...

    def stop(self) -> None: ...

    def get_paused(self) -> bool: ...

    def get_playing(self) -> bool: ...

class Chorus(pedalboard.Chorus):
    proportion_adjusted_depth: LinkedProportionConverter
    proportion_adjusted_feedback: LinkedProportionConverter
    proportion_adjusted_mix: LinkedProportionConverter

    def __init__(
            self,
            rate: int=1,
            depth: Numerical=25,
            center_delay: Numerical=0.007,
            feedback: Numerical=0,
            mix: Numerical=50,
            format: Literal["percentage", "decimal"]="percentage",
            depth_format: Literal["percentage", "decimal", None]=None,
            feedback_format: Literal["percentage", "decimal", None]=None,
            mix_format: Literal["percentage", "decimal", None]=None) -> None: ...

    def set_rate(self, rate: int) -> None: ...

    def get_rate(self) -> int: ...

    def set_center_delay(self, center_delay: Numerical) -> None: ...

    def get_center_delay(self) -> Numerical: ...

class Compressor(pedalboard.Compressor):
    def __init__(
            self,
            threshold: Numerical=0,
            ratio: Numerical=4,
            attack: Numerical=0.001,
            release: Numerical=0.1) -> None: ...

    def set_threshold(self, threshold: Numerical) -> None: ...

    def get_threshold(self) -> Numerical: ...

    def set_ratio(self, ratio: Numerical) -> None: ...

    def get_ratio(self) -> Numerical: ...

    def set_attack(self, attack: Numerical) -> None: ...

    def get_attack(self) -> Numerical: ...

    def set_release(self, release: Numerical) -> None: ...

    def get_release(self) -> Numerical: ...

class Convolution(pedalboard.Convolution):
    proportion_adjusted_mix: LinkedProportionConverter

    def __init__(
            self,
            impulse_response_filename: str,
            mix: Numerical=100,
            sample_rate: Union[int, None]=None,
            format: Literal["percentage", "decimal"]="percentage",
            mix_format: Literal["percentage", "decimal", None]=None) -> None: ...

    def set_impulse_response_filename(self, impulse_response: str) -> None: ...

    def get_impulse_response_filename(self) -> str: ...

    def set_sample_rate(self, sample_rate: Union[int, None]) -> None: ...

    def get_sample_rate(self) -> Union[int, None]: ...

class Delay(pedalboard.Delay):
    proportion_adjusted_feedback: LinkedProportionConverter
    proportion_adjusted_mix: LinkedProportionConverter

    def __init__(
            self,
            delay: Numerical=0.5,
            feedback: Numerical=0,
            mix: Numerical=50,
            format: Literal["percentage", "decimal"]="percentage",
            feedback_format: Literal["percentage", "decimal", None]=None,
            mix_format: Literal["percentage", "decimal", None]=None) -> None: ...

    def set_delay(self, delay: Numerical) -> None: ...

    def get_delay(self) -> Numerical: ...

class Phaser(pedalboard.Phaser):
    proportion_adjusted_depth = LinkedProportionConverter
    proportion_adjusted_feedback = LinkedProportionConverter
    proportion_adjusted_mix = LinkedProportionConverter

    def __init__(
            self,
            rate: int=1,
            depth: Numerical=50,
            center_frequency: int=1300,
            feedback: Numerical=0,
            mix: Numerical=50,
            format: Literal["percentage", "decimal"]="percentage",
            depth_format: Literal["percentage", "decimal", None]=None,
            feedback_format: Literal["percentage", "decimal", None]=None,
            mix_format: Literal["percentage", "decimal", None]=None) -> None: ...

    def set_rate(self, rate: int) -> None: ...

    def get_rate(self) -> int: ...

    def set_center_frequency(self, center_frequency: int) -> None: ...

    def get_center_frequency(self) -> int: ...

class Reverb(pedalboard.Reverb):
    proportion_adjusted_room_size = LinkedProportionConverter
    proportion_adjusted_damping = LinkedProportionConverter
    proportion_adjusted_wet_level = LinkedProportionConverter
    proportion_adjusted_dry_level = LinkedProportionConverter
    proportion_adjusted_width = LinkedProportionConverter

    def __init__(
            self,
            room_size: Numerical=100,
            damping: Numerical=50,
            wet_level: Numerical=33,
            dry_level: Numerical=40,
            width: Numerical=100,
            freeze_mode: bool=False,
            format: Literal["percentage", "decimal"]="percentage",
            room_size_format: Literal["percentage", "decimal", None]=None,
            damping_format: Literal["percentage", "decimal", None]=None,
            wet_level_format: Literal["percentage", "decimal", None]=None,
            dry_level_format: Literal["percentage", "decimal", None]=None,
            width_format: Literal["percentage", "decimal", None]=None) -> None: ...

    def set_freeze_mode(self, freeze_mode: bool) -> None: ...

    def get_freeze_mode(self) -> bool: ...


class Constants:
    DECIMAL: str
    PERCENTAGE: str

    VALUE: str
    UPDATING: str
    MANUALLY_SET: str

    WINDOWS: str
    LINUX: str
    MACOS: str
    JAVA: str
    ANDROID: str

    TKINTER_STYLE_BUTTON: str
    TKINTER_STYLE_CHECKBUTTON: str
    TKINTER_STYLE_COMBOBOX: str
    TKINTER_STYLE_ENTRY: str
    TKINTER_STYLE_FRAME: str
    TKINTER_STYLE_LABEL: str
    TKINTER_STYLE_LABELFRAME: str
    TKINTER_STYLE_MENUBUTTON: str
    TKINTER_STYLE_NOTEBOOK: str
    TKINTER_STYLE_PANEDWINDOW: str
    TKINTER_STYLE_HORIZONTAL_PROGRESSBAR: str
    TKINTER_STYLE_VERTICAL_PROGRESSBAR: str
    TKINTER_STYLE_RADIOBUTTON: str
    TKINTER_STYLE_HORIZONTAL_SCALE: str
    TKINTER_STYLE_VERTICAL_SCALE: str
    TKINTER_STYLE_HORIZONTAL_SCROLLBAR: str
    TKINTER_STYLE_VERTICAL_SCROLLBAR: str
    TKINTER_STYLE_SEPARATOR: str
    TKINTER_STYLE_SIZEGRIP: str
    TKINTER_STYLE_TREEVIEW: str


import os, shutil, pathlib

import send2trash

from pmma.core.py_src.Utility import Registry
import pmma.core.py_src.Error as Error

from pmma.build.Logger import Logger

class File:
    def __init__(self, file_path):
        self.logger = Logger()
        if Registry.passport_instance is None:
            self.logger.internal_log_error(
                23,
                "You have not created a passport for this application yet. \
This is used by PMMA and by your operating system to identify your application. \
Please use `pmma.Passport` to start creating a passport for your application.",
                True)

            raise Error.PassportNotInitializedError("Passport not initialized")

        self.file_path = str(pathlib.Path(file_path))

    def get_absolute_file_path(self):
        if not Registry.passport_instance.is_registered():
            self.logger.internal_log_error(
                35,
                "You have not registered this application yet. \
This is used by PMMA and by your operating system to identify your application. \
Please use `pmma.Passport.register` to register your application.",
                True)

            raise Error.PassportNotRegisteredError("Passport has not been registered")

        root = Registry.passport_instance.get_product_path()
        return root + os.sep + self.file_path

    def get_relative_file_path(self):
        return "." + os.sep + self.file_path

    def is_file_exists(self):
        return os.path.exists(self.get_absolute_file_path())

    def move(self, destination):
        src_path = self.get_absolute_file_path()
        root = Registry.passport_instance.get_product_path()
        destination = str(pathlib.Path(destination))
        destination = root + os.sep + destination + os.sep + os.path.basename(src_path)

        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                36,
                f"This file does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise FileNotFoundError

        if os.path.exists(destination):
            self.logger.internal_log_error(
                37,
                f"A file already exists at the specified location: \
'{destination}' with this name. Please ensure that you specify a file \
location that does not already exist.",
                True)

            raise FileExistsError

        shutil.move(src_path, destination)

        self.file_path = destination.replace(root + os.sep, "")

    def copy(self, destination):
        src_path = self.get_absolute_file_path()
        root = Registry.passport_instance.get_product_path()
        destination = str(pathlib.Path(destination))
        destination = root + os.sep + destination + os.sep + os.path.basename(src_path)
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                36,
                f"This file does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise FileNotFoundError

        if os.path.exists(destination):
            self.logger.internal_log_error(
                37,
                f"A file already exists at the specified location: \
'{destination}'. Please ensure that you specify a file location that does \
not already exist.",
                True)

            raise FileExistsError

        shutil.copy(src_path, destination)

    def delete(self):
        src_path = self.get_absolute_file_path()
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                36,
                f"This file does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise FileNotFoundError

        os.remove(src_path)

    def recycle(self):
        send2trash.send2trash(self.get_absolute_file_path())

    def rename(self, new_name):
        src_path = self.get_absolute_file_path()
        root = Registry.passport_instance.get_product_path()
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                36,
                f"This file does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise FileNotFoundError

        if os.path.exists(new_name):
            self.logger.internal_log_error(
                37,
                f"A file already exists at the specified location: \
'{new_name}' with this name. Please ensure that you specify a file \
location that does not already exist.",
                True)

            raise FileExistsError

        directory = os.path.dirname(src_path)
        extension = os.path.splitext(src_path)[1]

        new_path = os.path.join(directory, new_name + extension)

        os.rename(src_path, new_path)

        self.file_path = new_path.replace(root + os.sep, "")

class Directory:
    def __init__(self, directory_name):
        self.logger = Logger()
        if Registry.passport_instance is None:
            self.logger.internal_log_error(
                23,
                "You have not created a passport for this application yet. \
This is used by PMMA and by your operating system to identify your application. \
Please use `pmma.Passport` to start creating a passport for your application.",
                True)

            raise Error.PassportNotInitializedError("Passport not initialized")

        self.directory_name = str(pathlib.Path(directory_name))

    def get_absolute_path(self):
        if not Registry.passport_instance.is_registered():
            self.logger.internal_log_error(
                35,
                "You have not registered this application yet. \
This is used by PMMA and by your operating system to identify your application. \
Please use `pmma.Passport.register` to register your application.",
                True)

            raise Error.PassportNotRegisteredError("Passport has not been registered")

        root = Registry.passport_instance.get_product_path()

        return root + os.sep + self.directory_name

    def get_relative_path(self):
        return "." + os.sep + self.directory_name

    def is_directory_exists(self):
        return os.path.exists(self.get_absolute_path())

    def move(self, destination):
        src_path = self.get_absolute_path()
        root = Registry.passport_instance.get_product_path()
        destination = str(pathlib.Path(destination))
        destination = root + os.sep + destination
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                38,
                f"The directory does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise Error.DirectoryNotFoundError

        if os.path.exists(destination):
            self.logger.internal_log_error(
                39,
                f"A directory already exists at the specified location: \
'{destination}' with this name. Please ensure that you specify a directory \
location that does not already exist.",
                True)

            raise Error.DirectoryAlreadyExistsError

        shutil.move(src_path, destination)

        self.directory_name = destination.replace(root + os.sep, "")

    def copy(self, destination):
        src_path = self.get_absolute_path()
        root = Registry.passport_instance.get_product_path()
        destination = str(pathlib.Path(destination))
        destination = root + os.sep + destination
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                38,
                f"The directory does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise Error.DirectoryNotFoundError

        if os.path.exists(destination):
            self.logger.internal_log_error(
                39,
                f"A directory already exists at the specified location: \
'{destination}' with this name. Please ensure that you specify a directory \
location that does not already exist.",
                True)

            raise Error.DirectoryAlreadyExistsError

        shutil.copytree(src_path, destination)

    def delete(self):
        src_path = self.get_absolute_path()
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                38,
                f"The directory does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise Error.DirectoryNotFoundError

        shutil.rmtree(src_path)

    def recycle(self):
        send2trash.send2trash(self.get_absolute_path())

    def rename(self, new_name):
        src_path = self.get_absolute_path()
        root = Registry.passport_instance.get_product_path()
        directory = os.path.dirname(src_path)
        destination = os.path.join(directory, new_name)
        if not os.path.exists(src_path):
            self.logger.internal_log_error(
                38,
                f"The directory does not exist at the specified location: \
'{src_path}'. Please ensure that you have specified the correct path from your \
registered product location.",
                True)

            raise Error.DirectoryNotFoundError

        if os.path.exists(new_name):
            self.logger.internal_log_error(
                39,
                f"A directory already exists at the specified location: \
'{destination}' with this name. Please ensure that you specify a directory location that does \
not already exist.",
                True)

            raise Error.DirectoryAlreadyExistsError

        os.rename(src_path, destination)
        self.directory_name = destination.replace(root + os.sep, "")